/*
 *   BSD LICENSE
 *
 *   Copyright (C) EZchip Semiconductor Ltd. 2015.
 *
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the
 *       distribution.
 *     * Neither the name of EZchip Semiconductor nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _RTE_SPINLOCK_LOONGSON_H_
#define _RTE_SPINLOCK_LOONGSON_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <rte_common.h>
#include "generic/rte_spinlock.h"

#ifndef RTE_FORCE_INTRINSICS
static inline void
rte_spinlock_lock(rte_spinlock_t *sl)
{
    int temp = 0;
    __asm__ __volatile__(
        "1: ll.w %[temp], %[lock] \n"
        "   bnez %[temp], 1b \n"
        "   li.w %[temp], 1 \n"
        "   sc.w %[temp], %[lock] \n"
        "   beqz %[temp], 1b \n"
        : [lock] "+m" (sl->locked),
          [temp] "+&r"(temp)
        : "m"(sl->locked)
        : "memory"
        );
}

static inline void
rte_spinlock_unlock (rte_spinlock_t *sl)
{
    int temp = 0;
    __asm__ __volatile__(
        "1: ll.w %[temp], %[lock] \n"
        "   li.w %[temp], 0 \n"
        "   sc.w %[temp], %[lock] \n"
        "   beqz %[temp], 1b \n"
        : [lock] "+m" (sl->locked),
          [temp] "+&r"(temp)
        : "m"(sl->locked)
        : "memory"
        );
}

static inline int
rte_spinlock_trylock (rte_spinlock_t *sl)
{
    int temp = 0, res = 1;
    __asm__ __volatile__(
        "1: ll.w %[res], %[lock] \n"
        "   li.w %[temp], 1 \n"
        "   sc.w %[temp], %[lock] \n"
        "   beqz %[temp], 1b \n"
        : [lock] "+m" (sl->locked),
          [res] "=&r" (res),
          [temp] "+&r"(temp)
        : "m"(sl->locked)
        : "memory"
        );

    return res == 0;
}
#endif

static inline int rte_tm_supported(void)
{
	return 0;
}

static inline void
rte_spinlock_lock_tm(rte_spinlock_t *sl)
{
	rte_spinlock_lock(sl); /* fall-back */
}

static inline int
rte_spinlock_trylock_tm(rte_spinlock_t *sl)
{
	return rte_spinlock_trylock(sl);
}

static inline void
rte_spinlock_unlock_tm(rte_spinlock_t *sl)
{
	rte_spinlock_unlock(sl);
}

static inline void
rte_spinlock_recursive_lock_tm(rte_spinlock_recursive_t *slr)
{
	rte_spinlock_recursive_lock(slr); /* fall-back */
}

static inline void
rte_spinlock_recursive_unlock_tm(rte_spinlock_recursive_t *slr)
{
	rte_spinlock_recursive_unlock(slr);
}

static inline int
rte_spinlock_recursive_trylock_tm(rte_spinlock_recursive_t *slr)
{
	return rte_spinlock_recursive_trylock(slr);
}

#ifdef __cplusplus
}
#endif

#endif /* _RTE_SPINLOCK_LOONGSON_H_ */
